@@ -1,3 +1,7 @@
+# Copyright FuseSoC contributors
+# Licensed under the 2-Clause BSD License, see LICENSE for details.
+# SPDX-License-Identifier: BSD-2-Clause
+
 from collections import OrderedDict
 import importlib
 import logging
@@ -15,7 +19,7 @@ from fusesoc.capi1.plusargs import Plusargs
 logger = logging.getLogger(__name__)
 
 
-class FileSet(object):
+class FileSet:
     def __init__(self, name="", file=[], usage=[], private=False):
         self.name = name
         self.file = file
@@ -50,6 +54,10 @@ class Core:
     def __init__(self, core_file, cache_root=""):
         basename = os.path.basename(core_file)
         self.core_file = core_file
+
+        # Populated by CoreDB._solve(). TODO: Find a better solution for that.
+        self.direct_deps = []
+
         self.depend = []
         self.simulators = []
 
@@ -169,14 +177,14 @@ class Core:
     def get_files(self, flags={}):
         files = []
         flow = self._get_flow(flags)
-        usage = set([flow, flags["tool"]])
+        usage = {flow, flags["tool"]}
 
         for fs in self.file_sets:
             if (not fs.private or flags["is_toplevel"]) and (usage & set(fs.usage)):
-                files += fs.file
+                files += [vars(f) for f in fs.file]
         return files
 
-    def get_parameters(self, flags={}):
+    def get_parameters(self, flags={}, _parameters=None):
         self._debug("Getting parameters for flags '{}'".format(str(flags)))
         parameters = {}
         for k, v in self.parameter.items():
@@ -201,7 +209,7 @@ class Core:
                 "FILES_ROOT": files_root,
             }
             flow = self._get_flow(flags)
-            if flow is "sim":
+            if flow == "sim":
                 for s in ["pre_build_scripts", "pre_run_scripts", "post_run_scripts"]:
                     v = getattr(self.scripts, s)
                     if v:
@@ -209,7 +217,7 @@ class Core:
             # For backwards compatibility we only use the script from the
             # top-level core in synth flows. We also rename them here to match
             # the backend stages and set the SYSTEM_ROOT env var
-            elif flow is "synth" and flags["is_toplevel"]:
+            elif flow == "synth" and flags["is_toplevel"]:
                 env["SYSTEM_ROOT"] = self.files_root
                 v = self.scripts.pre_synth_scripts
                 if v:
@@ -333,7 +341,7 @@ class Core:
         return vpi
 
     def get_work_root(self, flags):
-        if self._get_flow(flags) is "synth":
+        if self._get_flow(flags) == "synth":
             s = "bld-"
         else:
             s = "sim-"
@@ -347,7 +355,7 @@ class Core:
         if os.path.exists(dst_dir):
             shutil.rmtree(dst_dir)
 
-        src_files = [f.name for f in self.get_files(flags)]
+        src_files = [f["name"] for f in self.get_files(flags)]
         if self.vpi and flags["tool"] in ["icarus", "modelsim", "rivierapro"]:
             src_files += [f.name for f in self.vpi.src_files + self.vpi.include_files]
         for section in self.get_scripts(dst_dir, flags).values():
@@ -401,9 +409,9 @@ class Core:
             ]:
                 flow = "synth"
         elif "target" in flags:
-            if flags["target"] is "synth":
+            if flags["target"] == "synth":
                 flow = "synth"
-            elif flags["target"] is "sim":
+            elif flags["target"] == "sim":
                 flow = "sim"
         return flow
 
